{#-
# ozi/templates/README.rst.j2
# Part of the OZI Project, under the Apache License v2.0 with LLVM Exceptions.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
-#}
.. OZI
  Classifier: License-Expression :: {{ project.license_expression }}
  Classifier: License-File :: {{ project.license_file }}

.. {%- for line in project.copyright_head.split('\n') %}
  {{ line }}
{%- endfor %}
{%- set numchar = project.name|length %}

{% for c in range(numchar) %}={% endfor %}
{{ project.name }}
{% for c in range(numchar) %}={% endfor %}

.. image:: https://raw.githubusercontent.com/sigstore/community/main/artwork/badge/sigstore_codesigned_purple.png
 :align: right
 :height: 140
 :target: https://www.sigstore.dev/
