{#- OZI 0.1
# ozi/templates/project.name/py.typed.j2
# Part of the OZI Project, under the Apache License v2.0 with LLVM Exceptions.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
# This file intentionally left empty. -#}