{#-
# ozi/templates/tests/meson.build.j2
# Part of the OZI Project, under the Apache License v2.0 with LLVM Exceptions.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
-#}
{% include 'project.PKG-INFO' %}